@@ -18,7 +18,7 @@
  
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 209281 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 230384 $")
 
 #include <string.h>
 #include <stdlib.h>
@@ -44,60 +44,105 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 209281 $")
 #include "asterisk/module.h"
 #include "asterisk/manager.h"
 
-static char *app_sndfax_name = "SendFAX";
-static char *app_sndfax_synopsis = "Send a FAX";
-static char *app_sndfax_desc = 
-"  SendFAX(filename[|options]):\n"
-"Send a given TIFF file to the channel as a FAX.\n"
-"The option string may contain zero or more of the following characters:\n"
-"     'a' - makes the application behave as an answering machine\n"
-"           The default behaviour is to behave as a calling machine.\n"
-"\n"
-"This application uses following variables:\n"
-"     LOCALSTATIONID to identify itself to the remote end.\n"
-"     LOCALHEADERINFO to generate a header line on each page.\n"
-"\n"
-"This application sets the following channel variables upon completion:\n"
-"     FAXSTATUS       - status of operation:\n"
-"                           SUCCESS | FAILED\n"
-"     FAXERROR        - Error when FAILED\n"
-"     FAXMODE         - Mode used:\n"
-"                           audio | T38\n"
-"     REMOTESTATIONID - CSID of the remote side.\n"
-"     FAXPAGES        - number of pages sent.\n"
-"     FAXBITRATE      - transmition rate.\n"
-"     FAXRESOLUTION   - resolution.\n"
-"\n"
-"Returns -1 in case of user hang up or any channel error.\n"
-"Returns 0 on success.\n";
+/*** DOCUMENTATION
+	<application name="SendFAX" language="en_US">
+		<synopsis>
+			Send a Fax
+		</synopsis>
+		<syntax>
+			<parameter name="filename" required="true">
+				<para>Filename of TIFF file to fax</para>
+			</parameter>
+			<parameter name="a" required="false">
+				<para>Makes the application behave as the answering machine</para>
+				<para>(Default behavior is as calling machine)</para>
+			</parameter>
+		</syntax>
+		<description>
+			<para>Send a given TIFF file to the channel as a FAX.</para>
+			<para>This application sets the following channel variables:</para>
+			<variablelist>
+				<variable name="LOCALSTATIONID">
+					<para>To identify itself to the remote end</para>
+				</variable>
+				<variable name="LOCALHEADERINFO">
+					<para>To generate a header line on each page</para>
+				</variable>
+				<variable name="FAXSTATUS">
+					<value name="SUCCESS"/>
+					<value name="FAILED"/>
+				</variable>
+				<variable name="FAXERROR">
+					<para>Cause of failure</para>
+				</variable>
+				<variable name="REMOTESTATIONID">
+					<para>The CSID of the remote side</para>
+				</variable>
+				<variable name="FAXPAGES">
+					<para>Number of pages sent</para>
+				</variable>
+				<variable name="FAXBITRATE">
+					<para>Transmission rate</para>
+				</variable>
+				<variable name="FAXRESOLUTION">
+					<para>Resolution of sent fax</para>
+				</variable>
+			</variablelist>
+		</description>
+	</application>
+	<application name="ReceiveFAX" language="en_US">
+		<synopsis>
+			Receive a Fax
+		</synopsis>
+		<syntax>
+			<parameter name="filename" required="true">
+				<para>Filename of TIFF file save incoming fax</para>
+			</parameter>
+			<parameter name="c" required="false">
+				<para>Makes the application behave as the calling machine</para> 
+				<para>(Default behavior is as answering machine)</para>
+			</parameter>
+		</syntax>
+		<description>
+			<para>Receives a FAX from the channel into the given filename 
+			overwriting the file if it already exists.</para>
+			<para>File created will be in TIFF format.</para>
 
+			<para>This application sets the following channel variables:</para>
+			<variablelist>
+				<variable name="LOCALSTATIONID">
+					<para>To identify itself to the remote end</para>
+				</variable>
+				<variable name="LOCALHEADERINFO">
+					<para>To generate a header line on each page</para>
+				</variable>
+				<variable name="FAXSTATUS">
+					<value name="SUCCESS"/>
+					<value name="FAILED"/>
+				</variable>
+				<variable name="FAXERROR">
+					<para>Cause of failure</para>
+				</variable>
+				<variable name="REMOTESTATIONID">
+					<para>The CSID of the remote side</para>
+				</variable>
+				<variable name="FAXPAGES">
+					<para>Number of pages sent</para>
+				</variable>
+				<variable name="FAXBITRATE">
+					<para>Transmission rate</para>
+				</variable>
+				<variable name="FAXRESOLUTION">
+					<para>Resolution of sent fax</para>
+				</variable>
+			</variablelist>
+		</description>
+	</application>
+
+ ***/
+
+static char *app_sndfax_name = "SendFAX";
 static char *app_rcvfax_name = "ReceiveFAX";
-static char *app_rcvfax_synopsis = "Receive a FAX";
-static char *app_rcvfax_desc = 
-"  ReceiveFAX(filename[|options]):\n"
-"Receives a fax from the channel into the given filename overwriting\n"
-"the file if it already exists. File created will have TIFF format.\n"
-"The option string may contain zero or more of the following characters:\n"
-"     'c' -- makes the application behave as a calling machine\n"
-"            The default behaviour is to behave as an answering machine.\n"
-"\n"
-"This application uses following variables:\n"
-"     LOCALSTATIONID to identify itself to the remote end.\n"
-"     LOCALHEADERINFO to generate a header line on each page.\n"
-"\n"
-"This application sets the following channel variables upon completion:\n"
-"     FAXSTATUS       - status of operation:\n"
-"                           SUCCESS | FAILED\n"
-"     FAXERROR        - Error when FAILED\n"
-"     FAXMODE         - Mode used:\n"
-"                           audio | T38\n"
-"     REMOTESTATIONID - CSID of the remote side.\n"
-"     FAXPAGES        - number of pages sent.\n"
-"     FAXBITRATE      - transmition rate.\n"
-"     FAXRESOLUTION   - resolution.\n"
-"\n"
-"Returns -1 in case of user hang up or any channel error.\n"
-"Returns 0 on success.\n";
 
 #define MAX_SAMPLES 240
 
@@ -334,8 +379,8 @@ static int transmit_audio(fax_session *s)
 							     .transcoding_jbig = 1,
 	};
 
-	/* if in receive mode, try to use T.38 */
-	if (!s->direction) {
+	/* if in called party mode, try to use T.38 */
+	if (s->caller_mode == FALSE) {
 		/* check if we are already in T.38 mode (unlikely), or if we can request
 		 * a switch... if so, request it now and wait for the result, rather
 		 * than starting an audio FAX session that will have to be cancelled
@@ -445,14 +490,30 @@ static int transmit_audio(fax_session *s)
 	ast_activate_generator(s->chan, &generator, &fax);
 
 	while (!s->finished) {
-		res = ast_waitfor(s->chan, 20);
-		if (res < 0)
-			break;
-		else if (res > 0)
-			res = 0;
+		inf = NULL;
 
-		inf = ast_read(s->chan);
-		if (inf == NULL) {
+		if ((res = ast_waitfor(s->chan, 25)) < 0) {
+			ast_debug(1, "Error waiting for a frame\n");
+			break;
+		}
+
+		/* Watchdog */
+		now = ast_tvnow();
+		if (ast_tvdiff_sec(now, start) > WATCHDOG_TOTAL_TIMEOUT || ast_tvdiff_sec(now, state_change) > WATCHDOG_STATE_TIMEOUT) {
+			ast_log(LOG_WARNING, "It looks like we hung. Aborting.\n");
+			res = -1;
+			break;
+		}
+
+		if (!res) {
+			/* There was timeout waiting for a frame. Loop around and wait again */
+			continue;
+		}
+
+		/* There is a frame available. Get it */
+		res = 0;
+
+		if (!(inf = ast_read(s->chan))) {
 			ast_debug(1, "Channel hangup\n");
 			res = -1;
 			break;
@@ -461,7 +522,7 @@ static int transmit_audio(fax_session *s)
 		ast_debug(10, "frame %d/%d, len=%d\n", inf->frametype, inf->subclass, inf->datalen);
 
 		/* Check the frame type. Format also must be checked because there is a chance
-		   that a frame in old format was already queued before we set chanel format
+		   that a frame in old format was already queued before we set channel format
 		   to slinear so it will still be received by ast_read */
 		if (inf->frametype == AST_FRAME_VOICE && inf->subclass == AST_FORMAT_SLINEAR) {
 			if (fax_rx(&fax, inf->data.ptr, inf->samples) < 0) {
@@ -470,8 +531,6 @@ static int transmit_audio(fax_session *s)
 				res = -1;
 				break;
 			}
-
-			/* Watchdog */
 			if (last_state != t30state->state) {
 				state_change = ast_tvnow();
 				last_state = t30state->state;
@@ -498,15 +557,6 @@ static int transmit_audio(fax_session *s)
 		}
 
 		ast_frfree(inf);
-		inf = NULL;
-
-		/* Watchdog */
-		now = ast_tvnow();
-		if (ast_tvdiff_sec(now, start) > WATCHDOG_TOTAL_TIMEOUT || ast_tvdiff_sec(now, state_change) > WATCHDOG_STATE_TIMEOUT) {
-			ast_log(LOG_WARNING, "It looks like we hung. Aborting.\n");
-			res = -1;
-			break;
-		}
 	}
 
 	ast_debug(1, "Loop finished, res=%d\n", res);
@@ -565,7 +615,8 @@ static int transmit_t38(fax_session *s)
 	memset(&t38, 0, sizeof(t38));
 	if (t38_terminal_init(&t38, s->caller_mode, t38_tx_packet_handler, s->chan) == NULL) {
 		ast_log(LOG_WARNING, "Unable to start T.38 termination.\n");
-		return -1;
+		res = -1;
+		goto disable_t38;
 	}
 
 	t38_set_max_datagram_size(t38state, s->t38parameters.max_ifp);
@@ -595,19 +646,31 @@ static int transmit_t38(fax_session *s)
 	now = start = state_change = ast_tvnow();
 
 	while (!s->finished) {
-
-		res = ast_waitfor(s->chan, 20);
-		if (res < 0)
+		inf = NULL;
+		if ((res = ast_waitfor(s->chan, 20)) < 0) {
 			break;
-		else if (res > 0)
-			res = 0;
+		}
 
 		last_frame = now;
 		now = ast_tvnow();
+		/* if nothing arrived, check the watchdog timers */
+		if (res == 0) {
+			if (ast_tvdiff_sec(now, start) > WATCHDOG_TOTAL_TIMEOUT || ast_tvdiff_sec(now, state_change) > WATCHDOG_STATE_TIMEOUT) {
+				ast_log(LOG_WARNING, "It looks like we hung. Aborting.\n");
+				res = -1;
+				break;
+			} else {
+				/* timers have not triggered, loop around to wait
+				 * again
+				 */
+				t38_terminal_send_timeout(&t38, ast_tvdiff_us(now, last_frame) / (1000000 / 8000));
+				continue;
+			}
+		}
+
 		t38_terminal_send_timeout(&t38, ast_tvdiff_us(now, last_frame) / (1000000 / 8000));
 
-		inf = ast_read(s->chan);
-		if (inf == NULL) {
+		if (!(inf = ast_read(s->chan))) {
 			ast_debug(1, "Channel hangup\n");
 			res = -1;
 			break;
@@ -617,8 +680,6 @@ static int transmit_t38(fax_session *s)
 
 		if (inf->frametype == AST_FRAME_MODEM && inf->subclass == AST_MODEM_T38) {
 			t38_core_rx_ifp_packet(t38state, inf->data.ptr, inf->datalen, inf->seqno);
-
-			/* Watchdog */
 			if (last_state != t30state->state) {
 				state_change = ast_tvnow();
 				last_state = t30state->state;
@@ -632,14 +693,6 @@ static int transmit_t38(fax_session *s)
 		}
 
 		ast_frfree(inf);
-		inf = NULL;
-
-		/* Watchdog */
-		if (ast_tvdiff_sec(now, start) > WATCHDOG_TOTAL_TIMEOUT || ast_tvdiff_sec(now, state_change) > WATCHDOG_STATE_TIMEOUT) {
-			ast_log(LOG_WARNING, "It looks like we hung. Aborting.\n");
-			res = -1;
-			break;
-		}
 	}
 
 	ast_debug(1, "Loop finished, res=%d\n", res);
@@ -650,6 +703,63 @@ static int transmit_t38(fax_session *s)
 	t30_terminate(t30state);
 	t38_terminal_release(&t38);
 
+disable_t38:
+	/* if we are not the caller, it's our job to shut down the T.38
+	 * session when the FAX transmisson is complete.
+	 */
+	if ((s->caller_mode == FALSE) &&
+	    (ast_channel_get_t38_state(s->chan) == T38_STATE_NEGOTIATED)) {
+		struct ast_control_t38_parameters t38_parameters = { .request_response = AST_T38_REQUEST_TERMINATE, };
+
+		if (ast_indicate_data(s->chan, AST_CONTROL_T38_PARAMETERS, &t38_parameters, sizeof(t38_parameters)) == 0) {
+			/* wait up to five seconds for negotiation to complete */
+			unsigned int timeout = 5000;
+			int ms;
+			
+			ast_debug(1, "Shutting down T.38 on %s\n", s->chan->name);
+			while (timeout > 0) {
+				ms = ast_waitfor(s->chan, 1000);
+				if (ms < 0) {
+					ast_log(LOG_WARNING, "something bad happened while channel '%s' was polling.\n", s->chan->name);
+					return -1;
+				}
+				if (!ms) {
+					/* nothing happened */
+					if (timeout > 0) {
+						timeout -= 1000;
+						continue;
+					} else {
+						ast_log(LOG_WARNING, "channel '%s' timed-out during the T.38 shutdown.\n", s->chan->name);
+						break;
+					}
+				}
+				if (!(inf = ast_read(s->chan))) {
+					return -1;
+				}
+				if ((inf->frametype == AST_FRAME_CONTROL) &&
+				    (inf->subclass == AST_CONTROL_T38_PARAMETERS) &&
+				    (inf->datalen == sizeof(t38_parameters))) {
+					struct ast_control_t38_parameters *parameters = inf->data.ptr;
+					
+					switch (parameters->request_response) {
+					case AST_T38_TERMINATED:
+						ast_debug(1, "Shut down T.38 on %s\n", s->chan->name);
+						break;
+					case AST_T38_REFUSED:
+						ast_log(LOG_WARNING, "channel '%s' refused to disable T.38\n", s->chan->name);
+						break;
+					default:
+						ast_log(LOG_ERROR, "channel '%s' failed to disable T.38\n", s->chan->name);
+						break;
+					}
+					ast_frfree(inf);
+					break;
+				}
+				ast_frfree(inf);
+			}
+		}
+	}
+
 	return res;
 }
 
@@ -813,8 +923,8 @@ static int load_module(void)
 {
 	int res ;
 
-	res = ast_register_application(app_sndfax_name, sndfax_exec, app_sndfax_synopsis, app_sndfax_desc);
-	res |= ast_register_application(app_rcvfax_name, rcvfax_exec, app_rcvfax_synopsis, app_rcvfax_desc);
+	res = ast_register_application_xml(app_sndfax_name, sndfax_exec);
+	res |= ast_register_application_xml(app_rcvfax_name, rcvfax_exec);
 
 	/* The default SPAN message handler prints to stderr. It is something we do not want */
 	span_set_message_handler(NULL);
