@@ -35,7 +35,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 211569 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 230510 $")
 
 #include "asterisk/paths.h"	/* use ast_config_AST_MONITOR_DIR */
 #include "asterisk/file.h"
@@ -46,42 +46,91 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 211569 $")
 #include "asterisk/app.h"
 #include "asterisk/channel.h"
 
+/*** DOCUMENTATION
+	<application name="MixMonitor" language="en_US">
+		<synopsis>
+			Record a call and mix the audio during the recording.  Use of StopMixMonitor is required
+			to guarantee the audio file is available for processing during dialplan execution.
+		</synopsis>
+		<syntax>
+			<parameter name="file" required="true" argsep=".">
+				<argument name="filename" required="true">
+					<para>If <replaceable>filename</replaceable> is an absolute path, uses that path, otherwise
+					creates the file in the configured monitoring directory from <filename>asterisk.conf.</filename></para>
+				</argument>
+				<argument name="extension" required="true" />
+			</parameter>
+			<parameter name="options">
+				<optionlist>
+					<option name="a">
+						<para>Append to the file instead of overwriting it.</para>
+					</option>
+					<option name="b">
+						<para>Only save audio to the file while the channel is bridged.</para>
+						<note><para>Does not include conferences or sounds played to each bridged party</para></note>
+						<note><para>If you utilize this option inside a Local channel, you must make sure the Local
+						channel is not optimized away. To do this, be sure to call your Local channel with the
+						<literal>/n</literal> option. For example: Dial(Local/start@mycontext/n)</para></note>
+					</option>
+					<option name="v">
+						<para>Adjust the <emphasis>heard</emphasis> volume by a factor of <replaceable>x</replaceable>
+						(range <literal>-4</literal> to <literal>4</literal>)</para>
+						<argument name="x" required="true" />
+					</option>
+					<option name="V">
+						<para>Adjust the <emphasis>spoken</emphasis> volume by a factor
+						of <replaceable>x</replaceable> (range <literal>-4</literal> to <literal>4</literal>)</para>
+						<argument name="x" required="true" />
+					</option>
+					<option name="W">
+						<para>Adjust both, <emphasis>heard and spoken</emphasis> volumes by a factor
+						of <replaceable>x</replaceable> (range <literal>-4</literal> to <literal>4</literal>)</para>
+						<argument name="x" required="true" />
+					</option>
+				</optionlist>
+			</parameter>
+			<parameter name="command">
+				<para>Will be executed when the recording is over.</para>
+				<para>Any strings matching <literal>^{X}</literal> will be unescaped to <variable>X</variable>.</para>
+				<para>All variables will be evaluated at the time MixMonitor is called.</para>
+			</parameter>
+		</syntax>
+		<description>
+			<para>Records the audio on the current channel to the specified file.</para>
+			<variablelist>
+				<variable name="MIXMONITOR_FILENAME">
+					<para>Will contain the filename used to record.</para>
+				</variable>
+			</variablelist>	
+		</description>
+		<see-also>
+			<ref type="application">Monitor</ref>
+			<ref type="application">StopMixMonitor</ref>
+			<ref type="application">PauseMonitor</ref>
+			<ref type="application">UnpauseMonitor</ref>
+		</see-also>
+	</application>
+	<application name="StopMixMonitor" language="en_US">
+		<synopsis>
+			Stop recording a call through MixMonitor, and free the recording's file handle.
+		</synopsis>
+		<syntax />
+		<description>
+			<para>Stops the audio recording that was started with a call to <literal>MixMonitor()</literal>
+			on the current channel.</para>
+		</description>
+		<see-also>
+			<ref type="application">MixMonitor</ref>
+		</see-also>
+	</application>
+		
+ ***/
+
 #define get_volfactor(x) x ? ((x > 0) ? (1 << x) : ((1 << abs(x)) * -1)) : 0
 
 static const char *app = "MixMonitor";
-static const char *synopsis = "Record a call and mix the audio during the recording";
-static const char *desc = ""
-"  MixMonitor(<file>.<ext>[,<options>[,<command>]]):\n"
-"Records the audio on the current channel to the specified file.\n"
-"If the filename is an absolute path, uses that path, otherwise\n"
-"creates the file in the configured monitoring directory from\n"
-"asterisk.conf.  Use of StopMixMonitor is required to guarantee\n"
-"the audio file is available for processing during dialplan execution.\n\n"
-"Valid options:\n"
-" a      - Append to the file instead of overwriting it.\n"
-" b      - Only save audio to the file while the channel is bridged.\n"
-"          Note: Does not include conferences or sounds played to each bridged\n"
-"                party.\n"
-"          Note: If you utilize this option inside a Local channel, you must\n"
-"                 make sure the Local channel is not optimized away. To do this,\n"
-"                 be sure to call your Local channel with the '/n' option.\n"
-"                 For example: Dial(Local/start@mycontext/n)\n"
-" v(<x>) - Adjust the heard volume by a factor of <x> (range -4 to 4)\n"	
-" V(<x>) - Adjust the spoken volume by a factor of <x> (range -4 to 4)\n"	
-" W(<x>) - Adjust the both heard and spoken volumes by a factor of <x>\n"
-"         (range -4 to 4)\n\n"	
-"<command> will be executed when the recording is over\n"
-"Any strings matching ^{X} will be unescaped to ${X}.\n"
-"All variables will be evaluated at the time MixMonitor is called.\n"
-"The variable MIXMONITOR_FILENAME will contain the filename used to record.\n"
-"";
 
 static const char *stop_app = "StopMixMonitor";
-static const char *stop_synopsis = "Stop recording a call through MixMonitor";
-static const char *stop_desc = ""
-"  StopMixMonitor():\n"
-"Stop recording a call through MixMonitor, and free the recording's file handle.\n"
-"";
 
 struct module_symbols *me;
 
@@ -140,18 +189,21 @@ struct mixmonitor_ds {
 	 * immediately during stop_mixmonitor or channel destruction. */
 	int fs_quit;
 	struct ast_filestream *fs;
+	struct ast_audiohook *audiohook;
 };
 
+ /*!
+  * \internal
+  * \pre mixmonitor_ds must be locked before calling this function
+  */
 static void mixmonitor_ds_close_fs(struct mixmonitor_ds *mixmonitor_ds)
 {
-	ast_mutex_lock(&mixmonitor_ds->lock);
 	if (mixmonitor_ds->fs) {
 		ast_closestream(mixmonitor_ds->fs);
 		mixmonitor_ds->fs = NULL;
 		mixmonitor_ds->fs_quit = 1;
 		ast_verb(2, "MixMonitor close filestream\n");
 	}
-	ast_mutex_unlock(&mixmonitor_ds->lock);
 }
 
 static void mixmonitor_ds_destroy(void *data)
@@ -160,6 +212,7 @@ static void mixmonitor_ds_destroy(void *data)
 
 	ast_mutex_lock(&mixmonitor_ds->lock);
 	mixmonitor_ds->chan = NULL;
+	mixmonitor_ds->audiohook = NULL;
 	mixmonitor_ds->destruction_ok = 1;
 	ast_cond_signal(&mixmonitor_ds->destruction_condition);
 	ast_mutex_unlock(&mixmonitor_ds->lock);
@@ -180,6 +233,20 @@ static struct ast_datastore_info mixmonitor_ds_info = {
 	.chan_fixup = mixmonitor_ds_chan_fixup,
 };
 
+static void destroy_monitor_audiohook(struct mixmonitor *mixmonitor)
+{
+	if (mixmonitor->mixmonitor_ds) {
+		ast_mutex_lock(&mixmonitor->mixmonitor_ds->lock);
+		mixmonitor->mixmonitor_ds->audiohook = NULL;
+		ast_mutex_unlock(&mixmonitor->mixmonitor_ds->lock);
+	}
+	/* kill the audiohook.*/
+	ast_audiohook_lock(&mixmonitor->audiohook);
+	ast_audiohook_detach(&mixmonitor->audiohook);
+	ast_audiohook_unlock(&mixmonitor->audiohook);
+	ast_audiohook_destroy(&mixmonitor->audiohook);
+}
+
 static int startmon(struct ast_channel *chan, struct ast_audiohook *audiohook) 
 {
 	struct ast_channel *peer = NULL;
@@ -220,10 +287,10 @@ static void *mixmonitor_thread(void *obj)
 
 	ast_verb(2, "Begin MixMonitor Recording %s\n", mixmonitor->name);
 
-	ast_audiohook_lock(&mixmonitor->audiohook);
-
 	fs = &mixmonitor->mixmonitor_ds->fs;
 
+	/* The audiohook must enter and exit the loop locked */
+	ast_audiohook_lock(&mixmonitor->audiohook);
 	while (mixmonitor->audiohook.status == AST_AUDIOHOOK_STATUS_RUNNING && !mixmonitor->mixmonitor_ds->fs_quit) {
 		struct ast_frame *fr = NULL;
 
@@ -235,6 +302,10 @@ static void *mixmonitor_thread(void *obj)
 		if (!(fr = ast_audiohook_read_frame(&mixmonitor->audiohook, SAMPLES_PER_FRAME, AST_AUDIOHOOK_DIRECTION_BOTH, AST_FORMAT_SLINEAR)))
 			continue;
 
+		/* audiohook lock is not required for the next block.
+		 * Unlock it, but remember to lock it before looping or exiting */
+		ast_audiohook_unlock(&mixmonitor->audiohook);
+
 		ast_mutex_lock(&mixmonitor->mixmonitor_ds->lock);
 		if (!ast_test_flag(mixmonitor, MUXFLAG_BRIDGED) || (mixmonitor->mixmonitor_ds->chan && ast_bridged_channel(mixmonitor->mixmonitor_ds->chan))) {
 			/* Initialize the file if not already done so */
@@ -266,26 +337,27 @@ static void *mixmonitor_thread(void *obj)
 
 		/* All done! free it. */
 		ast_frame_free(fr, 0);
+		ast_audiohook_lock(&mixmonitor->audiohook);
 	}
 
-	ast_audiohook_detach(&mixmonitor->audiohook);
 	ast_audiohook_unlock(&mixmonitor->audiohook);
-	ast_audiohook_destroy(&mixmonitor->audiohook);
-
 
+	/* Datastore cleanup.  close the filestream and wait for ds destruction */
+	ast_mutex_lock(&mixmonitor->mixmonitor_ds->lock);
 	mixmonitor_ds_close_fs(mixmonitor->mixmonitor_ds);
+	if (!mixmonitor->mixmonitor_ds->destruction_ok) {
+		ast_cond_wait(&mixmonitor->mixmonitor_ds->destruction_condition, &mixmonitor->mixmonitor_ds->lock);
+	}
+	ast_mutex_unlock(&mixmonitor->mixmonitor_ds->lock);
+
+	/* kill the audiohook */
+	destroy_monitor_audiohook(mixmonitor);
 
 	if (mixmonitor->post_process) {
 		ast_verb(2, "Executing [%s]\n", mixmonitor->post_process);
 		ast_safe_system(mixmonitor->post_process);
 	}
 
-	ast_mutex_lock(&mixmonitor->mixmonitor_ds->lock);
-	if (!mixmonitor->mixmonitor_ds->destruction_ok) {
-		ast_cond_wait(&mixmonitor->mixmonitor_ds->destruction_condition, &mixmonitor->mixmonitor_ds->lock);
-	}
-	ast_mutex_unlock(&mixmonitor->mixmonitor_ds->lock);
-
 	ast_verb(2, "End MixMonitor Recording %s\n", mixmonitor->name);
 	mixmonitor_free(mixmonitor);
 	return NULL;
@@ -312,6 +384,7 @@ static int setup_mixmonitor_ds(struct mixmonitor *mixmonitor, struct ast_channel
 
 	/* No need to lock mixmonitor_ds since this is still operating in the channel's thread */
 	mixmonitor_ds->chan = chan;
+	mixmonitor_ds->audiohook = &mixmonitor->audiohook;
 	datastore->data = mixmonitor_ds;
 
 	ast_channel_lock(chan);
@@ -353,6 +426,12 @@ static void launch_monitor_thread(struct ast_channel *chan, const char *filename
 		return;
 	}
 
+	/* Setup the actual spy before creating our thread */
+	if (ast_audiohook_init(&mixmonitor->audiohook, AST_AUDIOHOOK_TYPE_SPY, mixmonitor_spy_type)) {
+		mixmonitor_free(mixmonitor);
+		return;
+	}
+
 	/* Copy over flags and channel name */
 	mixmonitor->flags = flags;
 	if (setup_mixmonitor_ds(mixmonitor, chan)) {
@@ -369,12 +448,6 @@ static void launch_monitor_thread(struct ast_channel *chan, const char *filename
 	mixmonitor->filename = (char *) mixmonitor + sizeof(*mixmonitor) + strlen(chan->name) + 1;
 	strcpy(mixmonitor->filename, filename);
 
-	/* Setup the actual spy before creating our thread */
-	if (ast_audiohook_init(&mixmonitor->audiohook, AST_AUDIOHOOK_TYPE_SPY, mixmonitor_spy_type)) {
-		ast_free(mixmonitor);
-		return;
-	}
-
 	ast_set_flag(&mixmonitor->audiohook, AST_AUDIOHOOK_TRIGGER_SYNC);
 
 	if (readvol)
@@ -386,7 +459,7 @@ static void launch_monitor_thread(struct ast_channel *chan, const char *filename
 		ast_log(LOG_WARNING, "Unable to add '%s' spy to channel '%s'\n",
 			mixmonitor_spy_type, chan->name);
 		ast_audiohook_destroy(&mixmonitor->audiohook);
-		ast_free(mixmonitor);
+		mixmonitor_free(mixmonitor);
 		return;
 	}
 
@@ -478,13 +551,36 @@ static int stop_mixmonitor_exec(struct ast_channel *chan, void *data)
 {
 	struct ast_datastore *datastore = NULL;
 
-	/* closing the filestream here guarantees the file is avaliable to the dialplan
-	 * after calling StopMixMonitor */
-	if ((datastore = ast_channel_datastore_find(chan, &mixmonitor_ds_info, NULL))) {
-		mixmonitor_ds_close_fs(datastore->data);
-	}
-
+	ast_channel_lock(chan);
 	ast_audiohook_detach_source(chan, mixmonitor_spy_type);
+	if ((datastore = ast_channel_datastore_find(chan, &mixmonitor_ds_info, NULL))) {
+		struct mixmonitor_ds *mixmonitor_ds = datastore->data;
+
+		ast_mutex_lock(&mixmonitor_ds->lock);
+
+		/* closing the filestream here guarantees the file is avaliable to the dialplan
+	 	 * after calling StopMixMonitor */
+		mixmonitor_ds_close_fs(mixmonitor_ds);
+
+		/* The mixmonitor thread may be waiting on the audiohook trigger.
+		 * In order to exit from the mixmonitor loop before waiting on channel
+		 * destruction, poke the audiohook trigger. */
+		if (mixmonitor_ds->audiohook) {
+			ast_audiohook_lock(mixmonitor_ds->audiohook);
+			ast_cond_signal(&mixmonitor_ds->audiohook->trigger);
+			ast_audiohook_unlock(mixmonitor_ds->audiohook);
+			mixmonitor_ds->audiohook = NULL;
+		}
+
+		ast_mutex_unlock(&mixmonitor_ds->lock);
+
+		/* Remove the datastore so the monitor thread can exit */
+		if (!ast_channel_datastore_remove(chan, datastore)) {
+			ast_datastore_free(datastore);
+		}
+	}
+	ast_channel_unlock(chan);
+
 	return 0;
 }
 
@@ -494,7 +590,7 @@ static char *handle_cli_mixmonitor(struct ast_cli_entry *e, int cmd, struct ast_
 
 	switch (cmd) {
 	case CLI_INIT:
-		e->command = "mixmonitor [start|stop]";
+		e->command = "mixmonitor {start|stop} {<chan_name>} [args]";
 		e->usage =
 			"Usage: mixmonitor <start|stop> <chan_name> [args]\n"
 			"       The optional arguments are passed to the MixMonitor\n"
@@ -532,7 +628,7 @@ static int unload_module(void)
 {
 	int res;
 
-	ast_cli_unregister_multiple(cli_mixmonitor, sizeof(cli_mixmonitor) / sizeof(struct ast_cli_entry));
+	ast_cli_unregister_multiple(cli_mixmonitor, ARRAY_LEN(cli_mixmonitor));
 	res = ast_unregister_application(stop_app);
 	res |= ast_unregister_application(app);
 	
@@ -543,9 +639,9 @@ static int load_module(void)
 {
 	int res;
 
-	ast_cli_register_multiple(cli_mixmonitor, sizeof(cli_mixmonitor) / sizeof(struct ast_cli_entry));
-	res = ast_register_application(app, mixmonitor_exec, synopsis, desc);
-	res |= ast_register_application(stop_app, stop_mixmonitor_exec, stop_synopsis, stop_desc);
+	ast_cli_register_multiple(cli_mixmonitor, ARRAY_LEN(cli_mixmonitor));
+	res = ast_register_application_xml(app, mixmonitor_exec);
+	res |= ast_register_application_xml(stop_app, stop_mixmonitor_exec);
 
 	return res;
 }
