@@ -420,7 +420,7 @@ struct ast_channel {
 	struct ast_trans_pvt *readtrans;		/*!< Read translation path */
 	struct ast_audiohook_list *audiohooks;
 	struct ast_cdr *cdr;				/*!< Call Detail Record */
-	struct tone_zone *zone;			/*!< Tone zone as set in indications.conf or
+	struct ast_tone_zone *zone;			/*!< Tone zone as set in indications.conf or
 							     in the CHANNEL dialplan function */
 	struct ast_channel_monitor *monitor;		/*!< Channel monitoring */
 #ifdef HAVE_EPOLL
@@ -495,7 +495,8 @@ struct ast_channel {
 	union {
 		char unused_old_dtmfq[AST_MAX_EXTENSION];			/*!< (deprecated, use readq instead) Any/all queued DTMF characters */
 		struct {
-			struct ast_timer *timer;				/*!< timer object that provided timingfd */
+			struct ast_bridge *bridge;                                      /*!< Bridge this channel is participating in */
+			struct ast_timer *timer;					/*!< timer object that provided timingfd */
 		};
 	};
 
@@ -562,6 +563,8 @@ enum {
 	 *  bridge terminates, this will allow the hangup in the pbx loop to be run instead.
 	 *  */
 	AST_FLAG_BRIDGE_HANGUP_DONT = (1 << 18),
+	/*! This flag indicates whether the channel is in the channel list or not. */
+	AST_FLAG_IN_CHANNEL_LIST = (1 << 19),
 };
 
 /*! \brief ast_bridge_config flags */
@@ -942,7 +945,7 @@ int ast_hangup(struct ast_channel *chan);
  * \brief Softly hangup up a channel
  *
  * \param chan channel to be soft-hung-up
- * \param cause	Ast hangupcause for hangup
+ * \param reason an AST_SOFTHANGUP_* reason code
  *
  * Call the protocol layer, but don't destroy the channel structure
  * (use this if you are trying to
@@ -952,12 +955,13 @@ int ast_hangup(struct ast_channel *chan);
  *
  * \return Returns 0 regardless
  */
-int ast_softhangup(struct ast_channel *chan, int cause);
+int ast_softhangup(struct ast_channel *chan, int reason);
 
 /*! \brief Softly hangup up a channel (no channel lock)
  * \param chan channel to be soft-hung-up
- * \param cause	Ast hangupcause for hangup (see cause.h) */
-int ast_softhangup_nolock(struct ast_channel *chan, int cause);
+ * \param reason an AST_SOFTHANGUP_* reason code
+ */
+int ast_softhangup_nolock(struct ast_channel *chan, int reason);
 
 /*! \brief Check to see if a channel is needing hang up
  * \param chan channel on which to check for hang up
@@ -1316,6 +1320,18 @@ struct ast_channel *ast_get_channel_by_exten_locked(const char *exten, const cha
 struct ast_channel *ast_walk_channel_by_exten_locked(const struct ast_channel *chan, const char *exten,
 						     const char *context);
 
+/*! \brief Search for a channel based on the passed channel matching callback
+ * Search for a channel based on the specified is_match callback, and return the
+ * first channel that we match.  When returned, the channel will be locked.  Note
+ * that the is_match callback is called with the passed channel locked, and should
+ * return 0 if there is no match, and non-zero if there is.
+ * \param is_match callback executed on each channel until non-zero is returned, or we
+ *        run out of channels to search.
+ * \param data data passed to the is_match callback during each invocation.
+ * \return Returns the matched channel, or NULL if no channel was matched.
+ */
+struct ast_channel *ast_channel_search_locked(int (*is_match)(struct ast_channel *, void *), void *data);
+
 /*! ! \brief Waits for a digit
  * \param c channel to wait for a digit on
  * \param ms how many milliseconds to wait
