@@ -25,7 +25,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 203823 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 232011 $")
 
 #include <dirent.h>
 #include <sys/stat.h>
@@ -692,17 +692,36 @@ struct ast_filestream *ast_openvstream(struct ast_channel *chan, const char *fil
 	return NULL;
 }
 
+static struct ast_frame *read_frame(struct ast_filestream *s, int *whennext)
+{
+	struct ast_frame *fr, *new_fr;
+
+	if (!s || !s->fmt) {
+		return NULL;
+	}
+
+	if (!(fr = s->fmt->read(s, whennext))) {
+		return NULL;
+	}
+
+	if (!(new_fr = ast_frisolate(fr))) {
+		ast_frfree(fr);
+		return NULL;
+	}
+
+	if (new_fr != fr) {
+		ast_frfree(fr);
+		fr = new_fr;
+	}
+
+	return fr;
+}
+
 struct ast_frame *ast_readframe(struct ast_filestream *s)
 {
-	struct ast_frame *f = NULL;
-	int whennext = 0;	
-	if (s && s->fmt)
-		f = s->fmt->read(s, &whennext);
-	if (f) {
-		ast_set_flag(f, AST_FRFLAG_FROM_FILESTREAM);
-		ao2_ref(s, +1);
-	}
-	return f;
+	int whennext = 0;
+
+	return read_frame(s, &whennext);
 }
 
 enum fsread_res {
@@ -719,15 +738,13 @@ static enum fsread_res ast_readaudio_callback(struct ast_filestream *s)
 
 	while (!whennext) {
 		struct ast_frame *fr;
-		
-		if (s->orig_chan_name && strcasecmp(s->owner->name, s->orig_chan_name))
+
+		if (s->orig_chan_name && strcasecmp(s->owner->name, s->orig_chan_name)) {
 			goto return_failure;
-		
-		fr = s->fmt->read(s, &whennext);
-		if (fr) {
-			ast_set_flag(fr, AST_FRFLAG_FROM_FILESTREAM);
-			ao2_ref(s, +1);
 		}
+
+		fr = read_frame(s, &whennext);
+
 		if (!fr /* stream complete */ || ast_write(s->owner, fr) /* error writing */) {
 			if (fr) {
 				ast_log(LOG_WARNING, "Failed to write frame\n");
@@ -735,10 +752,12 @@ static enum fsread_res ast_readaudio_callback(struct ast_filestream *s)
 			}
 			goto return_failure;
 		} 
+
 		if (fr) {
 			ast_frfree(fr);
 		}
 	}
+
 	if (whennext != s->lasttimeout) {
 		if (s->owner->timingfd > -1) {
 			float samp_rate = (float) ast_format_rate(s->fmt->format);
@@ -782,11 +801,8 @@ static enum fsread_res ast_readvideo_callback(struct ast_filestream *s)
 	int whennext = 0;
 
 	while (!whennext) {
-		struct ast_frame *fr = s->fmt->read(s, &whennext);
-		if (fr) {
-			ast_set_flag(fr, AST_FRFLAG_FROM_FILESTREAM);
-			ao2_ref(s, +1);
-		}
+		struct ast_frame *fr = read_frame(s, &whennext);
+
 		if (!fr /* stream complete */ || ast_write(s->owner, fr) /* error writing */) {
 			if (fr) {
 				ast_log(LOG_WARNING, "Failed to write frame\n");
@@ -795,6 +811,7 @@ static enum fsread_res ast_readvideo_callback(struct ast_filestream *s)
 			s->owner->vstreamid = -1;
 			return FSREAD_FAILURE;
 		}
+
 		if (fr) {
 			ast_frfree(fr);
 		}
@@ -869,20 +886,23 @@ int ast_stream_rewind(struct ast_filestream *fs, off_t ms)
 
 int ast_closestream(struct ast_filestream *f)
 {
-	if (ast_test_flag(&f->fr, AST_FRFLAG_FROM_FILESTREAM)) {
-		/* If this flag is still set, it essentially means that the reference
-		 * count of f is non-zero. We can't destroy this filestream until
-		 * whatever is using the filestream's frame has finished.
-		 *
-		 * Since this was called, however, we need to remove the reference from
-		 * when this filestream was first allocated. That way, when the embedded
-		 * frame is freed, the refcount will reach 0 and we can finish destroying
-		 * this filestream properly.
-		 */
-		ao2_ref(f, -1);
-		return 0;
+	/* This used to destroy the filestream, but it now just decrements a refcount.
+	 * We need to force the stream to quit queuing frames now, because we might
+	 * change the writeformat, which could result in a subsequent write error, if
+	 * the format is different. */
+
+	/* Stop a running stream if there is one */
+	if (f->owner) {
+		if (f->fmt->format < AST_FORMAT_AUDIO_MASK) {
+			f->owner->stream = NULL;
+			AST_SCHED_DEL(f->owner->sched, f->owner->streamid);
+			ast_settimeout(f->owner, 0, NULL, NULL);
+		} else {
+			f->owner->vstream = NULL;
+			AST_SCHED_DEL(f->owner->sched, f->owner->vstreamid);
+		}
 	}
-	
+
 	ao2_ref(f, -1);
 	return 0;
 }
@@ -1314,17 +1334,6 @@ int ast_waitstream_exten(struct ast_channel *c, const char *context)
 		-1, -1, context);
 }
 
-void ast_filestream_frame_freed(struct ast_frame *fr)
-{
-	struct ast_filestream *fs;
-
-	ast_clear_flag(fr, AST_FRFLAG_FROM_FILESTREAM);
-
-	fs = (struct ast_filestream *) (((char *) fr) - offsetof(struct ast_filestream, fr));
-
-	ao2_ref(fs, -1);
-}
-
 /*
  * if the file name is non-empty, try to play it.
  * Return 0 if success, -1 if error, digit if interrupted by a digit.
@@ -1342,6 +1351,79 @@ int ast_stream_and_wait(struct ast_channel *chan, const char *file, const char *
 	return res;
 } 
 
+char *ast_format_str_reduce(char *fmts)
+{
+	struct ast_format *f;
+	struct ast_format *fmts_ptr[AST_MAX_FORMATS];
+	char *fmts_str[AST_MAX_FORMATS];
+	char *stringp, *type;
+	char *orig = fmts;
+	int i, j, x, first, found = 0;
+	int len = strlen(fmts) + 1;
+
+	if (AST_RWLIST_RDLOCK(&formats)) {
+		ast_log(LOG_WARNING, "Unable to lock format list\n");
+		return NULL;
+	}
+
+	stringp = ast_strdupa(fmts);
+
+	for (x = 0; (type = strsep(&stringp, "|")) && x < AST_MAX_FORMATS; x++) {
+		AST_RWLIST_TRAVERSE(&formats, f, list) {
+			if (exts_compare(f->exts, type)) {
+				found = 1;
+				break;
+			}
+		}
+
+		fmts_str[x] = type;
+		if (found) {
+			fmts_ptr[x] = f;
+		} else {
+			fmts_ptr[x] = NULL;
+		}
+	}
+	AST_RWLIST_UNLOCK(&formats);
+
+	first = 1;
+	for (i = 0; i < x; i++) {
+		/* ignore invalid entries */
+		if (!fmts_ptr[i]) {
+			ast_log(LOG_WARNING, "ignoring unknown format '%s'\n", fmts_str[i]);
+			continue;
+		}
+
+		/* special handling for the first entry */
+		if (first) {
+			fmts += snprintf(fmts, len, "%s", fmts_str[i]);
+			len -= (fmts - orig);
+			first = 0;
+			continue;
+		}
+
+		found = 0;
+		for (j = 0; j < i; j++) {
+			/* this is a duplicate */
+			if (fmts_ptr[j] == fmts_ptr[i]) {
+				found = 1;
+				break;
+			}
+		}
+
+		if (!found) {
+			fmts += snprintf(fmts, len, "|%s", fmts_str[i]);
+			len -= (fmts - orig);
+		}
+	}
+
+	if (first) {
+		ast_log(LOG_WARNING, "no known formats found in format list (%s)\n", orig);
+		return NULL;
+	}
+
+	return orig;
+}
+
 static char *handle_cli_core_show_file_formats(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
 #define FORMAT "%-10s %-10s %-20s\n"
@@ -1384,6 +1466,6 @@ struct ast_cli_entry cli_file[] = {
 
 int ast_file_init(void)
 {
-	ast_cli_register_multiple(cli_file, sizeof(cli_file) / sizeof(struct ast_cli_entry));
+	ast_cli_register_multiple(cli_file, ARRAY_LEN(cli_file));
 	return 0;
 }
